###
#File written by Michael Henson to calculate RPKMs
#You will need to provide three files: genome metadata, site metadata, and 
#file names. You will also need to provide the directory. If running in the bash script then 
#the file names with size and file column name should be automatically outputted. Both metadata 
#files must be provided. Blast output format used is 6. See line 33 for the headers. If you 
# do not want Density plots written out hash out line 110. 
###
#You will need the following libraries to run this code
library("cowplot")
library("stringr")
library("dplyr")
library("ggplot2")
library("data.table")

#Colorblind friendly palette http://dr-k-lo.blogspot.com/2013/07/a-color-blind-friendly-palette-for-r.html 
cbbPalette <- c("#F0E442", "#009E73", "#e79f00", "#9ad0f3", "#0072B2", "#D55E00", 
                "#CC79A7", "#000000")

#Function used to Calculate RPKMs To run rpkm(directory,foldername, metadata, site_metadata)
#Use quotes around each one to allow it to work (e.g. "Dropbox/LD12/metadata.csv")
rpkm<-function(directory,foldername, metadata, site_metadata){
  setwd(directory)
  foldername = read.table(foldername, col.names = c("size", "file_name"))
  metadata<-read.csv(metadata)
  #site metadata. Must match order of FWMG_countsall.txt
  site_metadata<-read.csv(site_metadata)
  #iteratively reads in file name
  for(d in 1:nrow(foldername)){
    file=as.character(foldername$file_name[d])
    MG= read.table(file, col.names =c('Read', 'Genome', 'PID', 'aln_len','mismatches','gaps','q_start','q_end','s_start','s_end', 'evalue','bitscore'))
    #determines the number of files in the folder
    MGlength=length(foldername$size)
    #Determines size (# of reads) of metagenome
    size=(foldername$size[d])/1000000
    #print(file)
    #print(size)
    #function to save ggplot outputs
    savePlot <- function(plot, outputfile) {
      pdf(outputfile)
      print(plot)
      dev.off()
    }
    #Used to calculate RPKM values betwee 85-100% identity
    PID= c(85, 90, 92, 95, 98, 100)
    for(num in PID){
      #filtering BLAST output files based on above percent IDs
      MG_PID=as.data.frame(filter(MG, PID >= num))
      #reformat the Data frame so that the new Data frame's # of factors equals the filtered size
      MG_PID[] <- lapply(MG_PID, function(x) if(is.factor(x)) factor(x) else x)
      #determines the number of genomes
      targets=length(metadata$Genome_name)
      #determines if the number of genomes that had hits equals the number of actual genomes used
      actual=length(unique(MG_PID$Genome))
      #if the filtered PID metagenome has no hits at that PID then skip it
      if (nrow(MG_PID)<1){
        next
      #runs this step if target genomes equals the number actual genomes
      } else if (targets == actual){
        #iteratively goes through the list of genomes to look at num of hits, size of genome, to calculate RPKM
        for (i in 1:targets){
          genomename=metadata$Genome_name[i]
          Lake=metadata$Lake[i]
          ID=metadata$ID[i]
          Label1=metadata$Label1[i]
          Label2=metadata$Label2[i]
          readcounts=nrow(filter(MG_PID, Genome== metadata$Genome_name[i]))
          genomelength=(metadata$Genome_length[i])/1000
          Subclade=metadata$Subclade[i]
          x4=readcounts/genomelength
          #calculate the RPKM
          xxx=x4*(1/size)
          #print(xxx)
          #print(ID)
          #print(genomename)
          #write out table that can be used for all downstream steps (currently does not printer headers to .csv file)
          write.table(data.frame(genomename,xxx,Lake,ID,Subclade, file, Label1, Label2, readcounts, genomelength, size),file=as.character(paste(num,"RPKM_output.csv",sep="_")),append=TRUE, col.names = FALSE, quote = FALSE, sep = ",")
        }
        #Not all the metagenomes have hits to all 45 genomes used so need to filter the metadata file to contain only genonmes with hits
      } else if (targets != actual) {
        target = actual
        #gets the names of the genomes that are actually in the metagenome when target does not equal actual
        actualgenomes=as.vector(unique(MG_PID$Genome))
        #creates new data frame that contains only the genomes found in the metagenome 
        newmetadata<-metadata[metadata$Genome_name %in% actualgenomes,]
        #the above step gives you a new data frame but the number of factors is kept.Therefore, we need to rewrite it so that only the used factors are listed
        newmetadata[] <- lapply(newmetadata, function(x) if(is.factor(x)) factor(x) else x)
        for (z in 1:target){
          genomename=newmetadata$Genome_name[z]
          #print(genomename)
          Lake=newmetadata$Lake[z]
          ID=newmetadata$ID[z]
          Label1=newmetadata$Label1[z]
          Label2=newmetadata$Label2[z]
          Subclade=newmetadata$Subclade[z]
          #print(ID)
          genomelength=(newmetadata$Genome_length[z])/1000
          #calculate the RP
          readcounts=nrow(filter(MG_PID, Genome==newmetadata$Genome_name[z]))
          #calculate the RPKM
          x4=readcounts/genomelength
          xxx=x4*(1/size)
          #print(xxx)
          write.table(data.frame(genomename,xxx,Lake,ID,Subclade,file, Label1, Label2, readcounts, genomelength, size),file=as.character(paste(num,"RPKM_output.csv",sep="_")),append=TRUE, col.names = FALSE, quote = FALSE, sep = ",")
        }
      }
    }
    #creates data table for the density plots
    data <- left_join(MG, metadata,by = c("Genome"="Genome_name"))
    #creates outname that plot will be saved as
    outputFile= paste(foldername$file_name[d],"density.pdf",sep="_")  
    #ggplot function to create
    plotden<- function(data, name){
      ggplot(data, aes(x=PID, colour=Label1)) +geom_density(alpha=0.5, size=1)+
        theme_bw() + scale_y_continuous(limits = c(0,0.30))+
        xlab('Percent Identity') + 
        ylab('Density') +
        labs(title=name)+ scale_color_manual(values = cbbPalette)+
        theme(axis.title=element_text(size=20,face="bold"),legend.title=element_text(size=15), legend.text= element_text(size=15),axis.text = element_text(size=15),axis.text.x  = element_text(angle=90, vjust=0.5))
    }
    plot<-plotden(data,as.character(paste(site_metadata$Site[d],"Salinity=", site_metadata$Salinity[d],sep=" ")))
    savePlot(plot, outputFile)
    dev.off
  }
#Now will read in the outputted rpkm files from above and iterate over them to create RPKM plots  
#This is set up specifcally to run SAR11 LD12 organisms. To change to be general remove filter step
Foo= c(85, 90, 92, 95, 98, 100)
  for (PI in Foo){
    rpkm_data<-read.table(as.character(paste(PI,"RPKM_output.csv",sep="_")), col.names =c('X', 'genome_fasta', 'RPKM', 'Lake','ID', 'Subclade', 'file', 'Label1', 'Label2', 'readcounts', 'genomelength', 'size'), sep=',')
    rpkm_plot_data<- left_join(rpkm_data, site_metadata, by = c("file"="input_name"))
    rpkm_plot_data_LD12=as.data.frame(filter(rpkm_plot_data, Subclade == 'IIIb'))
    rpkm_plot_data_LD12[] <- lapply(rpkm_plot_data_LD12, function(x) if(is.factor(x)) factor(x) else x)
    rpkm_dt<-data.table(rpkm_plot_data_LD12)
    rpkm_plot_data_LD12<-as.data.frame(rpkm_dt[, list(RPKM=sum(RPKM)), by=list(Label1, Site, Salinity, raw_file, EnvType)])
    rpkm_plot_data_LD12<-rpkm_plot_data_LD12[order(rpkm_plot_data_LD12$Salinity),]
    rpkm_plot_data_LD12$raw_file<-factor(rpkm_plot_data_LD12$raw_file, levels = unique(rpkm_plot_data_LD12$raw_file))
    plotrpkm<-function(rpkm_plot_data_LD12, site_metadata){
      ggplot(rpkm_plot_data_LD12, aes(raw_file, Label1)) + geom_point(aes(size=rpkm_plot_data_LD12$RPKM, color=rpkm_plot_data_LD12$EnvType))+
        xlab('Site') + 
        ylab('LD12 Micro cluster') +
        labs(title=paste("RPKM Plot",PI, sep=" "), color='Lake', size='RPKM')+ scale_color_manual(values = cbbPalette)+
        theme(axis.title=element_text(size=20,face="bold"),axis.text.x  = element_text(angle=90, vjust=0.5))
    }
    plot2<-plotrpkm(rpkm_plot_data_LD12, site_metadata)
    savePlot(plot2, as.character(paste(PI, "RPKM_output.pdf",sep="_")))
    dev.off
  }
}

